@@ -26,6 +26,9 @@ from logdetective.constants import (
     USER_ROLE_DEFAULT,
 )
 
+from logdetective.extractors import Extractor, DrainExtractor, CSGrepExtractor
+from logdetective.utils import check_csgrep
+
 
 class BuildLog(BaseModel):
     """Model of data submitted to API."""
@@ -88,6 +91,24 @@ class EmojiHook(BaseModel):
     merge_request: EmojiMergeRequest = Field(default=None)
 
 
+class SnippetAnalysis(BaseModel):
+    """Model of snippet analysis from LLM."""
+
+    text: str = Field(description="Analysis of log snippet contents.")
+
+
+class RatedSnippetAnalysis(SnippetAnalysis):
+    """Model for rated snippet analysis. This model is used to generate
+    json schema for inference with structured output."""
+
+    relevance: int = Field(
+        ge=0,
+        le=100,
+        description="Estimate of likelyhood that snippet contains an error, "
+        "with 0 standing for completely unlikely, 100 for absolutely certain.",
+    )
+
+
 class Explanation(BaseModel):
     """Model of snippet or general log explanation from Log Detective"""
 
@@ -95,6 +116,7 @@ class Explanation(BaseModel):
     logprobs: Optional[List[Dict]] = None
 
     def __str__(self):
+        """Return text of the Explanation"""
         return self.text
 
 
@@ -106,7 +128,7 @@ class AnalyzedSnippet(BaseModel):
     line_number: location of snippet in original log
     """
 
-    explanation: Explanation
+    explanation: SnippetAnalysis | RatedSnippetAnalysis
     text: str
     line_number: int
 
@@ -225,18 +247,59 @@ class InferenceConfig(BaseModel):  # pylint: disable=too-many-instance-attribute
 class ExtractorConfig(BaseModel):
     """Model for extractor configuration of logdetective server."""
 
-    context: bool = True
     max_clusters: int = 8
     verbose: bool = False
+    max_snippet_len: int = 2000
+    csgrep: bool = False
+
+    _extractors: List[Extractor] = []
+
+    def _setup_extractors(self):
+        """Initialize extractors with common settings."""
+        self._extractors = [
+            DrainExtractor(
+                verbose=self.verbose,
+                max_snippet_len=self.max_snippet_len,
+                max_clusters=self.max_clusters,
+            )
+        ]
+
+        if self.csgrep:
+            self._extractors.append(
+                CSGrepExtractor(
+                    verbose=self.verbose,
+                    max_snippet_len=self.max_snippet_len,
+                )
+            )
 
     def __init__(self, data: Optional[dict] = None):
-        super().__init__()
+        super().__init__(data=data)
+
         if data is None:
+            self._setup_extractors()
             return
 
-        self.context = data.get("context", True)
         self.max_clusters = data.get("max_clusters", 8)
         self.verbose = data.get("verbose", False)
+        self.max_snippet_len = data.get("max_snippet_len", 2000)
+        self.csgrep = data.get("csgrep", False)
+
+        self._setup_extractors()
+
+    def get_extractors(self) -> List[Extractor]:
+        """Return list of initialized extractors, each will be applied in turn
+        on original log text to retrieve snippets."""
+        return self._extractors
+
+    @field_validator("csgrep", mode="after")
+    @classmethod
+    def validate_csgrep(cls, value: bool) -> bool:
+        """Verify that csgrep is available if requested."""
+        if not check_csgrep():
+            raise ValueError(
+                "Requested csgrep extractor but `csgrep` binary is not in the PATH"
+            )
+        return value
 
 
 class GitLabInstanceConfig(BaseModel):  # pylint: disable=too-many-instance-attributes
@@ -244,6 +307,7 @@ class GitLabInstanceConfig(BaseModel):  # pylint: disable=too-many-instance-attr
 
     name: str = None
     url: str = None
+    # Path to API of the gitlab instance, assuming `url` as prefix.
     api_path: str = None
     api_token: str = None
 
@@ -439,6 +503,7 @@ class GeneralConfig(BaseModel):
     devmode: bool = False
     sentry_dsn: HttpUrl | None = None
     collect_emojis_interval: int = 60 * 60  # seconds
+    top_k_snippets: int = 0
 
     def __init__(self, data: Optional[dict] = None):
         super().__init__()
@@ -452,6 +517,7 @@ class GeneralConfig(BaseModel):
         self.collect_emojis_interval = data.get(
             "collect_emojis_interval", 60 * 60
         )  # seconds
+        self.top_k_snippets = data.get("top_k_snippets", 0)
 
 
 class Config(BaseModel):
@@ -460,6 +526,7 @@ class Config(BaseModel):
     log: LogConfig = LogConfig()
     inference: InferenceConfig = InferenceConfig()
     snippet_inference: InferenceConfig = InferenceConfig()
+    # TODO(jpodivin): Extend to work with multiple extractor configs
     extractor: ExtractorConfig = ExtractorConfig()
     gitlab: GitLabConfig = GitLabConfig()
     koji: KojiConfig = KojiConfig()
