@@ -7,6 +7,7 @@ from typing import Annotated
 from io import BytesIO
 
 import matplotlib
+import matplotlib.figure
 import matplotlib.pyplot
 from fastapi import (
     FastAPI,
@@ -34,21 +35,15 @@ from logdetective.server.database.models.exceptions import (
 
 import logdetective.server.database.base
 
-from logdetective.utils import (
-    compute_certainty,
-    format_snippets,
-    prompt_to_messages,
-)
-
-from logdetective.server.config import SERVER_CONFIG, PROMPT_CONFIG, LOG
+from logdetective.server.config import SERVER_CONFIG, LOG
 from logdetective.server.koji import (
     get_failed_log_from_task as get_failed_log_from_koji_task,
 )
 from logdetective.remote_log import RemoteLog
 from logdetective.server.llm import (
-    mine_logs,
     perform_staged_analysis,
-    submit_text,
+    perfrom_analysis,
+    perform_analyis_stream,
 )
 from logdetective.server.gitlab import process_gitlab_job_event
 from logdetective.server.metric import track_request, add_new_metrics, update_metrics
@@ -111,34 +106,36 @@ async def get_http_session(request: Request) -> aiohttp.ClientSession:
     return request.app.http
 
 
-def requires_token_when_set(authentication: Annotated[str | None, Header()] = None):
+def requires_token_when_set(authorization: Annotated[str | None, Header()] = None):
     """
-    FastAPI Depend function that expects a header named Authentication
+    FastAPI Depend function that expects a header named Authorization
 
     If LOGDETECTIVE_TOKEN env var is set, validate the client-supplied token
     otherwise ignore it
     """
     if not API_TOKEN:
-        LOG.info("LOGDETECTIVE_TOKEN env var not set, authentication disabled")
+        LOG.info("LOGDETECTIVE_TOKEN env var not set, authorization disabled")
         # no token required, means local dev environment
         return
-    token = None
-    if authentication:
+    if authorization:
         try:
-            token = authentication.split(" ", 1)[1]
-        except (ValueError, IndexError):
+            token = authorization.split(" ", 1)[1]
+        except (ValueError, IndexError) as ex:
             LOG.warning(
-                "Authentication header has invalid structure (%s), it should be 'Bearer TOKEN'",
-                authentication,
+                "Authorization header has invalid structure '%s', it should be 'Bearer TOKEN'",
+                authorization,
             )
             # eat the exception and raise 401 below
-            token = None
+            raise HTTPException(
+                status_code=401,
+                detail=f"Invalid authorization, HEADER '{authorization}' not valid.",
+            ) from ex
         if token == API_TOKEN:
             return
-    LOG.info(
-        "LOGDETECTIVE_TOKEN env var is set (%s), clien token = %s", API_TOKEN, token
-    )
-    raise HTTPException(status_code=401, detail=f"Token {token} not valid.")
+        LOG.info("Provided token '%s' does not match expected value.", token)
+        raise HTTPException(status_code=401, detail=f"Token '{token}' not valid.")
+    LOG.error("No authorization header provided but LOGDETECTIVE_TOKEN env var is set")
+    raise HTTPException(status_code=401, detail="No token provided.")
 
 
 app = FastAPI(dependencies=[Depends(requires_token_when_set)], lifespan=lifespan)
@@ -157,31 +154,8 @@ async def analyze_log(
     """
     remote_log = RemoteLog(build_log.url, http_session)
     log_text = await remote_log.process_url()
-    log_summary = mine_logs(log_text)
-    log_summary = format_snippets(log_summary)
-    messages = prompt_to_messages(
-        PROMPT_CONFIG.prompt_template.format(log_summary),
-        PROMPT_CONFIG.default_system_prompt,
-        SERVER_CONFIG.inference.system_role,
-        SERVER_CONFIG.inference.user_role,
-    )
-    response = await submit_text(
-        messages,
-        inference_cfg=SERVER_CONFIG.inference,
-    )
-    certainty = 0
 
-    if response.logprobs is not None:
-        try:
-            certainty = compute_certainty(response.logprobs)
-        except ValueError as ex:
-            LOG.error("Error encountered while computing certainty: %s", ex)
-            raise HTTPException(
-                status_code=400,
-                detail=f"Couldn't compute certainty with data:\n{response.logprobs}",
-            ) from ex
-
-    return Response(explanation=response, response_certainty=certainty)
+    return await perfrom_analysis(log_text)
 
 
 @app.post("/analyze/staged", response_model=StagedResponse)
@@ -351,9 +325,7 @@ async def analyze_koji_task(task_id: int, koji_instance_config: KojiInstanceConf
     # Notify any callbacks that the analysis is complete.
     for callback in koji_instance_config.get_callbacks(task_id):
         LOG.info("Notifying callback %s of task %d completion", callback, task_id)
-        asyncio.create_task(
-            send_koji_callback(callback, task_id)
-        )
+        asyncio.create_task(send_koji_callback(callback, task_id))
 
     # Now that it's sent, we can clear the callbacks for this task.
     koji_instance_config.clear_callbacks(task_id)
@@ -398,20 +370,8 @@ async def analyze_log_stream(
     """
     remote_log = RemoteLog(build_log.url, http_session)
     log_text = await remote_log.process_url()
-    log_summary = mine_logs(log_text)
-    log_summary = format_snippets(log_summary)
-    messages = prompt_to_messages(
-        PROMPT_CONFIG.prompt_template.format(log_summary),
-        PROMPT_CONFIG.default_system_prompt,
-        SERVER_CONFIG.inference.system_role,
-        SERVER_CONFIG.inference.user_role,
-    )
     try:
-        stream = submit_text(
-            messages,
-            inference_cfg=SERVER_CONFIG.inference,
-            stream=True,
-        )
+        stream = perform_analyis_stream(log_text)
     except aiohttp.ClientResponseError as ex:
         raise HTTPException(
             status_code=400,
@@ -419,9 +379,6 @@ async def analyze_log_stream(
             f"[{ex.status}] {ex.message}",
         ) from ex
 
-    # we need to figure out a better response here, this is how it looks rn:
-    # b'data: {"choices":[{"finish_reason":"stop","index":0,"delta":{}}],
-    #   "created":1744818071,"id":"chatcmpl-c9geTxNcQO7M9wR...
     return StreamingResponse(stream)
 
 
@@ -711,7 +668,7 @@ async def collect_emoji_task():
             instance.url,
             datetime.datetime.now(datetime.timezone.utc),
         )
-        await collect_emojis(instance.get_connection(), TimePeriod(weeks="54"))
+        await collect_emojis(instance.get_connection(), TimePeriod(weeks=54))
         LOG.info(
             "Collect emoji feedback finished at %s",
             datetime.datetime.now(datetime.timezone.utc),
